/* This file is part of GNOME Games. License: GPL-3.0+. */

.solid-background {
	background-color: @theme_bg_color;
}

.content-view {
	background-color: @theme_base_color;
}

.savestate-row {
	padding: 0;
}

.savestate-thumbnail {
	color: rgba(255, 255, 255, 0.5);
	background: rgba (0, 0, 0, .5);
	border: 1px solid rgba (0, 0, 0, .5);
	border-radius: 5px;
}

.savestate-name {
	font-size: large;
}

.savestate-date {
	opacity: 0.5;
	font-size: small;
}

.savestates-list {
	border-left: none;
	border-right: none;
}

gamesgamethumbnail {
	background-color: mix (@theme_base_color, @theme_bg_color, 0.5);
	border-width: 1px;
	border-style: solid;
	border-radius: 2px;
	border-color: @theme_bg_color;
	color: @theme_fg_color;
}

gamescollectioniconview flowboxchild {
	min-width: 128px;
}

gamescollectioniconview.large flowboxchild {
	min-width: 256px;
}

gamesflashbox {
  background: rgba (255, 255, 255, 0.5);
}

/* Styles from libhandy example, this should be kept in sync with this:
 * https://source.puri.sm/Librem5/libhandy/blob/master/examples/style.css
 */

stacksidebar list {
	border-left-width: 0px;
	border-right-width: 0px;
}

.list-icon {
	opacity: 0.7;
}

.combo-list-box {
	background: none;
}
